
#include <stdio.h>
#include <stdlib.h>
#include "..\..\..\bmp.h"
#include "..\..\..\j2k-codec.h"

int main(int ArgC, char **ArgV)
{
 void *j2k_image; char Key[] = "Your registration key";
 int   j2k_ver, j2k_err, width, height, components, resolutions=0, rLevel, rWidth, rHeight; 
 char  options[100];
 unsigned char *buffer=0; int size, bpp, pitch=0; 

 if(ArgC < 2)
 {
  printf("\n Usage:   d_AdvancedDecode.exe <filename.j2k> [<resolution level>]\n");
  printf("\n Example: d_AdvancedDecode.exe test.j2k 1\n\n");
  system("pause"); 
  return -1; 
 }

 /* Get the DLL version and make sure it is at least 1.1 */

 j2k_ver = J2K_getVersion(); if(j2k_ver < 0x110000) return -1;


 /* Remove the demo version restrictions */

 J2K_Unlock(Key); 


 /* Open file, parse header and get basic info about the image */

 j2k_image = J2K_OpenFile(ArgV[1]);
 if(!j2k_image)
 { 
  printf("\n File '%s' opening error: %s.\n", ArgV[1], J2K_getLastErrorStr()); return -1; 
 }

 j2k_err = J2K_GetInfo(j2k_image, &width, &height, &components);
 if(j2k_err != J2KERR_SUCCESS)
 {
  printf("\n J2K_GetInfo() error: %s.\n", J2K_getErrorStr(j2k_err)); 
  J2K_Close(j2k_image);
  return -1; 
 }

 printf("\n Image info: %dx%d, %d component(s).\n", width, height, components);

 /* Now get the information about resolution levels */

 j2k_err = J2K_GetResolutions(j2k_image, &resolutions);
 if(j2k_err != J2KERR_SUCCESS) printf("\n J2K_GetResolutions() error: %s.\n", J2K_getErrorStr(j2k_err));

 printf("\n %d resolution(s) available.\n", resolutions);


 /* Get optional resolution level parameter from the command line */

 rLevel = 0; /* By default, the whole image will be decoded */

 if(ArgC > 2) 
 {
  rLevel = atoi(ArgV[2]);
  if(rLevel < 0)
  {
   printf("\n Incorrect resolution %d was specified! Decoding the whole image instead...\n", rLevel); 
   rLevel = 0;
  }
 }


 /* If this function fails it will fill rWidth and rHeight with the image size. 
    So for simplicity we won't handle the failure here. */

 J2K_GetResolutionDimensions(j2k_image, rLevel, &rWidth, &rHeight); 

 printf("\n Decoding resolution %d (%dx%d)...\n", rLevel, rWidth, rHeight);


 /* Let's allocate the buffer ourselves */

 bpp = (components==1) ? 1 : 4;

 pitch = rWidth * bpp; size = pitch * rHeight * bpp;

 buffer = malloc(size); 

 if(!buffer) printf("\n Couldn't allocate %d bytes!\n", size);
 else
 {
  /* Prepare the options string */

  sprintf(options, "bpp=%d,rl=%d", bpp, rLevel); 

  /* Decode the resolution with the correct bpp */

  j2k_err = J2K_Decode(j2k_image, &buffer, &size, options, &pitch);
  if(j2k_err != J2KERR_SUCCESS) printf("\n Decoding error: %s.\n", J2K_getErrorStr(j2k_err));
  else
  {
   /* Finally save the decoded image as BMP file */

   SaveAsBMP("decoded.bmp", buffer, rWidth, rHeight, bpp);

   printf("\n Done!\n");
  }

  free(buffer);
 }

 /* Clean up */

 J2K_Close(j2k_image);

 return 0;
}
